------------------------------------------------------------------------------
-- MakeMeGetMyMain - Alarms you when something happens to your character
------------------------------------------------------------------------------
-- Options.lua - Options
--
-- Author: Expelliarm5s / January 2023 / All Rights Reserved
--
-- Version 1.1.23
------------------------------------------------------------------------------
-- luacheck: ignore 212 globals DLAPI
-- luacheck: globals AceGUIWidgetLSMlists, max line length 160, ignore 212

local addonName, addon = ...
local Options = addon:NewModule("Options", "AceConsole-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local LSM = LibStub("LibSharedMedia-3.0")
local WidgetLists = AceGUIWidgetLSMlists
--------------------------------------------------------------------------------------------------------

--  Options
Options.defaults = {
	profile = {
	},
	global = {
		closeOnEscape = false,
		windowTransparent = false,
		windowFullMode = false,
		windowSmallMode = false,
		enabled = true,
		timeout = "8",

		Fight = false,
		SoundFight = "Simon Small Blue",

		Kill = true,
		SoundKill = "Simon Small Blue",

		Fatigue = true,
		SoundFatigue = "Simon Small Blue",

		Whisper = true,
		SoundWhisper = "Simon Small Blue",

		Move = false,
		SoundMove = "Simon Small Blue",

		Loot = true,
		SoundLoot = "Simon Small Blue",

		ItemSold = true,
		SoundItemSold = "Simon Small Blue",

		Say = true,
		SoundSay = "Simon Small Blue",
		SayWordList = "multiboxer, ticket",
		LootItemIDs = "163573, 163574, 163575, 163576, 163131",

		BagSpace = true,
		SoundBagSpace = "Simon Small Blue",
		BagSpaceSlots = "2",
		minimap = {
			hide = false,
		},
	},
}

------------------------------------------------------------------------------
-- Debug Stuff

function Options:DebugPrintf(...)
	if addon.isDebug then
		local status, res = pcall(format, ...)
		if status then
			addon:DebugLog("Opt~" .. res)
		end
	end
end

------------------------------------------------------------------------------
-- Addon Loading / Player Login/Logout

function Options:Login()
	Options:DebugPrintf("Login()")

	Options.isClassic = (WOW_PROJECT_ID ~= WOW_PROJECT_MAINLINE)

	-- see https://wow.tools/files/#search=&page=1&sort=0&desc=asc
	if LSM then
		if not Options.isClassic then
			LSM:Register("sound", "Default", 567482)
			LSM:Register("sound", "Small Chain", 567577)
			LSM:Register("sound", "Bell Toll Alliance", 566564)
			LSM:Register("sound", "Bell Toll Horde", 565853)
			LSM:Register("sound", "Auction Window Close", 567499)
			LSM:Register("sound", "Quest Failed", 567459)
			LSM:Register("sound", "Fel Nova", 568582)
			LSM:Register("sound", "Simon Large Blue", 566076)
			LSM:Register("sound", "Simon Small Blue", 567002)
			LSM:Register("sound", "Portcullis Close", 566240)
			LSM:Register("sound", "PvP Flag Taken", 569200)
			LSM:Register("sound", "Sound Cannon", 566101)
			LSM:Register("sound", "Alarm 2", 567399)
		else
			LSM:Register("sound", "Default", SOUNDKIT.AUCTION_WINDOW_OPEN)
			LSM:Register("sound", "Small Chain", SOUNDKIT.PUT_DOWN_SMALL_CHAIN)
			LSM:Register("sound", "AH Bell", SOUNDKIT.AUCTION_WINDOW_CLOSE)
			LSM:Register("sound", "Simon Small Blue", SOUNDKIT.RAID_WARNING)
		end
	end
end

function Options.PlaySound(key)
	if LSM and key and LSM:Fetch("sound", key) then
		local sound = LSM:Fetch("sound", key)
		if sound == "Interface\\Quiet.ogg" then
			-- nix
			Options:DebugPrintf("Playing/1: silence")
		else
			if tonumber(sound) then
				if Options.isClassic then
					Options:DebugPrintf("PlaySound/2: %s", tostring(sound))
					PlaySound(sound, "master")
				else
					Options:DebugPrintf("PlaySoundFile/3: %s", tostring(sound))
					PlaySoundFile(sound, "master")
				end
			elseif type(sound) == "string" then
				if sound:match("^Sound") then
					Options:DebugPrintf("PlaySound/4: std sound")
					PlaySound(SOUNDKIT.AUCTION_WINDOW_OPEN, "master")
				else
					Options:DebugPrintf("PlaySoundFile/5: %s", sound)
					PlaySoundFile(sound, "master")
				end
			else
				Options:DebugPrintf("PlaySound/6: std sound")
				PlaySound(SOUNDKIT.AUCTION_WINDOW_OPEN, "master")
			end
		end
	else
		Options:DebugPrintf("PlaySound/7: std sound")
		PlaySound(SOUNDKIT.AUCTION_WINDOW_OPEN, "master")
	end
end

function Options.GetOptions(uiType, uiName, appName)
	if appName == addonName then

		local wowV, wowP = GetBuildInfo()
		local wowVersion = "|nGame: WoW (ID " .. WOW_PROJECT_ID .. "), Version: " .. wowV .. ", Build: " .. wowP

		local soundWidget = "LSM30_Sound"
		if Options.isClassic then
			soundWidget = "LSM30_SoundClassic"
		end

		local options = {
			type = "group",
			name = addon.METADATA.NAME .. " (" .. addon.METADATA.VERSION .. ")",
			get = function(info)
					return addon.db.global[info[#info]] or ""
				end,
			set = function(info, value)
					addon.db.global[info[#info]] = value
					addon:Update()
				end,
			args = {
				desc1a = {
					type = "description",
					order = 0,
					name = "|cff99ccff-: by " .. GetAddOnMetadata(addonName, "Author") .. " :-|r|n|n" .. GetAddOnMetadata(addonName, "Notes") ..
					"|n|n" .. L["  Reset Options to Default Values with /mmgmm reset"] ..
					"|n" .. L["  Import/Export Options with /mmgmm export"],
					fontSize = "medium",
				},
				desc1b = {
					type = "description",
					order = 0.01,
					name = wowVersion,
				},
				header01 = {
					type = "header",
					name = L["Alarm Window"],
					order = 0.3,
				},
				closeOnEscape = {
					type = "toggle",
					name = L["Close Window with Escape"],
					desc = L["If checked, the alarm window can be closed by pressing 'Esc'."],
					order = 0.5,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				windowFullMode = {
					type = "toggle",
					name = L["Show Bigger Window"],
					desc = L["If checked, the window is bigger and looting may be disabled."],
					order = 0.6,
					width = "normal",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				windowTransparent = {
					type = "toggle",
					name = L["Transparent Window Background"],
					desc = L["If checked, the background of the alarm window is transparent."],
					order = 0.65,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				windowSmallMode = {
					type = "toggle",
					name = L["Show Smaller Window"],
					desc = L["If checked, the window is smaller."],
					order = 0.7,
					width = "normal",
					get = function(info) return addon.db.global[info[#info]] end,
				},
				timeout = {
					type = "input",
					name = L["Window Timout"],
					desc = L["If > 0 the window is closed after this many seconds."],
					width = "full",
					order = 0.71,
					get = function(info)
						return addon.db.global.timeout
					end,
					set = function(info, value)
						if value then
							if tonumber(value) then
								addon.db.global.timeout = value
							else
								addon.db.global.timeout = "0"
							end
						end
					end,
				},
			},
		}

		local cnt = 1
		for m in pairs(addon.modules) do
			if addon[m].ModuleName then
				cnt = cnt + 1
				local o = {
					type = "header",
					name = format(L["Detection Module: |cffffffff%s|r"], m),
					order = cnt,
				}
				options.args["header" .. m] = o

				if addon[m].TriggerDesc then
					cnt = cnt + 1
					o = {
						type = "description",
						name = addon[m].TriggerDesc and addon[m].TriggerDesc() or "",
						order = cnt,
					}
					options.args["desc" .. m] = o
				end

				cnt = cnt + 1
				o = {
					type = "toggle",
					name = format(L["Enable Module |cfffed100%s|r"], m),
					desc = format(L["If checked, alarms from module %s are displayed."], m),
					order = cnt,
					width = "double",
					get = function(info) return addon.db.global[info[#info]] end,
					set = function(info, value)
						addon.db.global[info[#info]] = value
						addon:Update()
					end,
				}
				options.args[m] = o

				if addon.db.global[m] then
					cnt = cnt + 1
					o = {
						type = "select",
						name = L["Alarm Sound:"],
						desc = L["Select the sound that is played if the module fires an alarm. Disable sound with 'None'."],
						order = cnt,
						width = "normal",
						dialogControl = soundWidget,
						values = WidgetLists.sound,
						get = function() return addon.db.global["Sound" .. m] end,
						set = function(_, value)
							-- value = "Default"
							addon.db.global["Sound" .. m] = value
							Options:DebugPrintf("LSM: Key=%s", value)
							if LSM and value and LSM:Fetch("sound", value) then
								local sound = LSM:Fetch("sound", value)
								if sound then
									if type(sound) == "string" then
										Options:DebugPrintf("LSM: String=%s", sound)
									end
									if type(sound) == "number" then
										Options:DebugPrintf("LSM: Number=%s", tostring(sound))
									end
								end
							end
						end,
					}
					options.args["soundoptions" .. m] = o

					if addon[m].GetOptions then
						cnt = cnt + 1
						o = addon[m].GetOptions()
						options.args["options" .. m] = o
						options.args["options" .. m].order = cnt
					end
				end
			end
		end

		return options
	end
end

-- EOF
